@@ -1,22 +1,28 @@
 package Sub::Uplevel;
-
 use 5.006;
 use strict;
-our $VERSION = '0.21_01';
-$VERSION = eval $VERSION;
+# ABSTRACT: apparently run a function in a higher stack frame
+our $VERSION = '0.23_01'; # VERSION
+
+# Frame check global constant
+our $CHECK_FRAMES;
+BEGIN {
+  $CHECK_FRAMES = !! $CHECK_FRAMES;
+}
+use constant CHECK_FRAMES => $CHECK_FRAMES;
 
 # We must override *CORE::GLOBAL::caller if it hasn't already been 
 # overridden or else Perl won't see our local override later.
 
 if ( not defined *CORE::GLOBAL::caller{CODE} ) {
-    *CORE::GLOBAL::caller = \&_normal_caller;
+  *CORE::GLOBAL::caller = \&_normal_caller;
 }
 
 # modules to force reload if ":aggressive" is specified
 my @reload_list = qw/Exporter Exporter::Heavy/;
 
 sub import {
-  no strict 'refs';
+  no strict 'refs'; ## no critic
   my ($class, @args) = @_;
   for my $tag ( @args, 'uplevel' ) {
     if ( $tag eq 'uplevel' ) {
@@ -42,18 +48,116 @@ sub _force_reload {
     require $m if delete $INC{$m};
   }
 }
-  
+
+
+# @Up_Frames -- uplevel stack
+# $Caller_Proxy -- whatever caller() override was in effect before uplevel
+our (@Up_Frames, $Caller_Proxy);
+
+sub _apparent_stack_height {
+    my $height = 1; # start above this function 
+    while ( 1 ) {
+        last if ! defined scalar $Caller_Proxy->($height);
+        $height++;
+    }
+    return $height - 1; # subtract 1 for this function
+}
+
+sub uplevel {
+    # Backwards compatible version of "no warnings 'redefine'"
+    my $old_W = $^W;
+    $^W = 0;
+
+    # Update the caller proxy if the uplevel override isn't in effect
+    local $Caller_Proxy = *CORE::GLOBAL::caller{CODE}
+        if *CORE::GLOBAL::caller{CODE} != \&_uplevel_caller;
+    local *CORE::GLOBAL::caller = \&_uplevel_caller;
+
+    # Restore old warnings state
+    $^W = $old_W;
+
+    if ( CHECK_FRAMES and $_[0] >= _apparent_stack_height() ) {
+      require Carp;
+      Carp::carp("uplevel $_[0] is more than the caller stack");
+    }
+
+    local @Up_Frames = (shift, @Up_Frames );
+
+    my $function = shift;
+    return $function->(@_);
+}
+
+sub _normal_caller (;$) { ## no critic Prototypes
+    my ($height) = @_;
+    $height++;
+    my @caller = CORE::caller($height);
+    if ( CORE::caller() eq 'DB' ) {
+        # Oops, redo picking up @DB::args
+        package DB;
+        @caller = CORE::caller($height);
+    }
+
+    return if ! @caller;                  # empty
+    return $caller[0] if ! wantarray;     # scalar context
+    return @_ ? @caller : @caller[0..2];  # extra info or regular
+}
+
+sub _uplevel_caller (;$) { ## no critic Prototypes
+    my $height = $_[0] || 0;
+
+    # shortcut if no uplevels have been called
+    # always add +1 to CORE::caller (proxy caller function)
+    # to skip this function's caller
+    return $Caller_Proxy->( $height + 1 ) if ! @Up_Frames;
+
+
+    my $saw_uplevel = 0;
+    my $adjust = 0;
+
+    # walk up the call stack to fight the right package level to return;
+    # look one higher than requested for each call to uplevel found
+    # and adjust by the amount found in the Up_Frames stack for that call.
+    # We *must* use CORE::caller here since we need the real stack not what 
+    # some other override says the stack looks like, just in case that other
+    # override breaks things in some horrible way
+
+    for ( my $up = 0; $up <= $height + $adjust; $up++ ) {
+        my @caller = CORE::caller($up + 1); 
+        if( defined $caller[0] && $caller[0] eq __PACKAGE__ ) {
+            # add one for each uplevel call seen
+            # and look into the uplevel stack for the offset
+            $adjust += 1 + $Up_Frames[$saw_uplevel];
+            $saw_uplevel++;
+        }
+    }
+
+    # For returning values, we pass through the call to the proxy caller
+    # function, just at a higher stack level
+    my @caller = $Caller_Proxy->($height + $adjust + 1);
+    if ( CORE::caller() eq 'DB' ) {
+        # Oops, redo picking up @DB::args
+        package DB;
+        @caller = $Sub::Uplevel::Caller_Proxy->($height + $adjust + 1);
+    }
+
+    return if ! @caller;                  # empty
+    return $caller[0] if ! wantarray;     # scalar context
+    return @_ ? @caller : @caller[0..2];  # extra info or regular
+}
+
+
+1;
+
+__END__
+=pod
+
 =head1 NAME
 
 Sub::Uplevel - apparently run a function in a higher stack frame
 
-=begin wikidoc
+=head1 VERSION
 
-= VERSION
-
-This documentation describes version %%VERSION%%
-
-=end wikidoc
+version 0.23_01
 
 =head1 SYNOPSIS
 
@@ -107,79 +211,23 @@ you can do this:
         return @out;
     }
 
-C<uplevel> will issue a warning if C<$num_frames> is more than the current call
-stack depth.
+C<uplevel> has the ability to issue a warning if C<$num_frames> is more than
+the current call stack depth, although this warning is disabled and compiled
+out by default as the check is relatively expensive.
 
-=cut
+To enable the check for debugging or testing, you should set the global
+C<$Sub::Uplevel::CHECK_FRAMES> to true before loading Sub::Uplevel for the
+first time as follows:
 
-# @Up_Frames -- uplevel stack
-# $Caller_Proxy -- whatever caller() override was in effect before uplevel
-our (@Up_Frames, $Caller_Proxy);
-
-sub _apparent_stack_height {
-    my $height = 1; # start above this function 
-    while ( 1 ) {
-        last if ! defined scalar $Caller_Proxy->($height);
-        $height++;
-    }
-    return $height - 1; # subtract 1 for this function
-}
-
-sub uplevel {
-    my($num_frames, $func, @args) = @_;
+    #!/usr/bin/perl
     
-    # backwards compatible version of "no warnings 'redefine'"
-    my $old_W = $^W;
-    $^W = 0;
-
-    # Update the caller proxy if the uplevel override isn't in effect
-    local $Caller_Proxy = *CORE::GLOBAL::caller{CODE}
-        if *CORE::GLOBAL::caller{CODE} != \&_uplevel_caller;
-    local *CORE::GLOBAL::caller = \&_uplevel_caller;
-    
-    # restore old warnings state
-    $^W = $old_W;
-
-    if ( $num_frames >= _apparent_stack_height() ) {
-      require Carp;
-      Carp::carp("uplevel $num_frames is more than the caller stack");
+    BEGIN {
+        $Sub::Uplevel::CHECK_FRAMES = 1;
     }
+    use Sub::Uplevel;
 
-    local @Up_Frames = ($num_frames, @Up_Frames );
-    
-    return $func->(@args);
-}
-
-sub _normal_caller (;$) { ## no critic Prototypes
-    my $height = $_[0];
-    $height++;
-    if ( CORE::caller() eq 'DB' ) {
-        # passthrough the @DB::args trick
-        package DB;
-        if( wantarray and !@_ ) {
-            return (CORE::caller($height))[0..2];
-        }
-        else {
-            return CORE::caller($height);
-        }
-    }
-    else {
-        if( wantarray and !@_ ) {
-            return (CORE::caller($height))[0..2];
-        }
-        else {
-            return CORE::caller($height);
-        }
-    }
-}
-
-sub _uplevel_caller (;$) { ## no critic Prototypes
-    my $height = $_[0] || 0;
-
-    # shortcut if no uplevels have been called
-    # always add +1 to CORE::caller (proxy caller function)
-    # to skip this function's caller
-    return $Caller_Proxy->( $height + 1 ) if ! @Up_Frames;
+Setting or changing the global after the module has been loaded will have
+no effect.
 
 =begin _private
 
@@ -228,51 +276,6 @@ in the call stack, i.e. the requested height plus any uplevel adjustments
 found during the search
 
 =end _dagolden
-        
-=cut
-
-    my $saw_uplevel = 0;
-    my $adjust = 0;
-
-    # walk up the call stack to fight the right package level to return;
-    # look one higher than requested for each call to uplevel found
-    # and adjust by the amount found in the Up_Frames stack for that call.
-    # We *must* use CORE::caller here since we need the real stack not what 
-    # some other override says the stack looks like, just in case that other
-    # override breaks things in some horrible way
-
-    for ( my $up = 0; $up <= $height + $adjust; $up++ ) {
-        my @caller = CORE::caller($up + 1); 
-        if( defined $caller[0] && $caller[0] eq __PACKAGE__ ) {
-            # add one for each uplevel call seen
-            # and look into the uplevel stack for the offset
-            $adjust += 1 + $Up_Frames[$saw_uplevel];
-            $saw_uplevel++;
-        }
-    }
-
-    # For returning values, we pass through the call to the proxy caller
-    # function, just at a higher stack level
-    my @caller;
-    if ( CORE::caller() eq 'DB' ) {
-        # passthrough the @DB::args trick
-        package DB;
-        @caller = $Sub::Uplevel::Caller_Proxy->($height + $adjust + 1);
-    }
-    else {
-        @caller = $Caller_Proxy->($height + $adjust + 1);
-    }
-
-    if( wantarray ) {
-        if( !@_ ) {
-            @caller = @caller[0..2];
-        }
-        return @caller;
-    }
-    else {
-        return $caller[0];
-    }
-}
 
 =back
 
@@ -293,7 +296,6 @@ around functions and they wouldn't be aware they've been wrapped.
 
 If this code frightens you B<you should not use this module.>
 
-
 =head1 BUGS and CAVEATS
 
 Well, the bad news is uplevel() is about 5 times slower than a normal
@@ -311,7 +313,7 @@ CORE::GLOBAL::caller, you are probably asking for trouble.
 You B<should> load Sub::Uplevel as early as possible within your program.  As
 with all CORE::GLOBAL overloading, the overload will not affect modules that
 have already been compiled prior to the overload.  One module that often is
-unavoidably loaded prior to Sub::Uplevel is Exporter.  To forceably recompile
+unavoidably loaded prior to Sub::Uplevel is Exporter.  To forcibly recompile
 Exporter (and Exporter::Heavy) after loading Sub::Uplevel, use it with the
 ":aggressive" tag:
 
@@ -334,20 +336,6 @@ dinner table.
 
 Thanks to Brent Welch, Damian Conway and Robin Houston.
 
-=head1 AUTHORS
-
-David A Golden E<lt>dagolden@cpan.orgE<gt> (current maintainer)
-
-Michael G Schwern E<lt>schwern@pobox.comE<gt> (original author)
-
-=head1 LICENSE
-
-Original code Copyright (c) 2001 to 2007 by Michael G Schwern.
-Additional code Copyright (c) 2006 to 2008 by David A Golden.
-
-This program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
-
 See http://www.perl.com/perl/misc/Artistic.html
 
 =head1 SEE ALSO
@@ -355,6 +343,45 @@ See http://www.perl.com/perl/misc/Artistic.html
 PadWalker (for the similar idea with lexicals), Hook::LexWrap, 
 Tcl's uplevel() at http://www.scriptics.com/man/tcl8.4/TclCmd/uplevel.htm
 
+=for :stopwords cpan testmatrix url annocpan anno bugtracker rt cpants kwalitee diff irc mailto metadata placeholders
+
+=head1 SUPPORT
+
+=head2 Bugs / Feature Requests
+
+Please report any bugs or feature requests through the issue tracker
+at L<http://rt.cpan.org/Public/Dist/Display.html?Name=Sub-Uplevel>.
+You will be notified automatically of any progress on your issue.
+
+=head2 Source Code
+
+This is open source software.  The code repository is available for
+public review and contribution under the terms of the license.
+
+L<https://github.com/dagolden/sub-uplevel>
+
+  git clone https://github.com/dagolden/sub-uplevel.git
+
+=head1 AUTHORS
+
+=over 4
+
+=item *
+
+Michael Schwern <mschwern@cpan.org>
+
+=item *
+
+David Golden <dagolden@cpan.org>
+
+=back
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2012 by Michael Schwern and David Golden.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
 =cut
 
-1;
